import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Graphical representation of a MIDI note.
 * 
 * @author  Andersson, Andreen, Lanneskog, Pehrson
 * @version 1
 */
 
public class MooNoteElement extends JPanel {

	private MooTrackView mtv;
	private MooNote note;
	private int columns;
	private boolean selected;
	private Rectangle pitchRect, veloRect;
	public static final Color bgColor = new Color(160, 218, 255);
	private String notePitch;
	private String noteVelocity;
	private JPopupMenu popup;
	private JMenuItem popupRemove, popupProp;

	/** 
	 * Creates a new note element.
	 * @param mn	the note that will be graphically represented
	 * @param rows	the number of rows that the note will occupy
	 */
	public MooNoteElement (MooTrackView parent, MooNote mn) {
		mtv = parent;
		note = mn;
		calculateString();
		columns = mn.getDuration() / (Moosique.getSequence().getResolution() / 4);
		setBorder(BorderFactory.createLineBorder(Color.black));
		setBackground(bgColor);
		addMouseListener(new MAdapter());

		// Defines coordinates.
		pitchRect = new Rectangle(0, 0, 15, 10);
		veloRect = new Rectangle(20, 0, 40, 10);

		// Creates pop-up menu.
		popup = new JPopupMenu();
		PopupListener pList = new PopupListener();
		popupProp = new JMenuItem("Preferences...");
		popupProp.addActionListener(pList);
		popup.add(popupProp);

		popupRemove = new JMenuItem("Remove");
		popupRemove.addActionListener(pList);
		popup.add(popupRemove);

	}

	/** 
	 * Returns true if the current NoteElement is selected, otherwise false.
	 * @return if the element is selected
	 */
	public boolean isSelected() {
		return selected;
	}

	/** 
	 * Selects the current NoteElement.
	 * @param state	if the element should be selected
	 */
	public void setSelected(boolean state) {
		selected = state;
	}

	/**
	 *
	 */
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		if (!(g instanceof Graphics2D)) return;
		Graphics2D g2 = (Graphics2D)g;
		g2.setFont(new Font("Helvetica", Font.PLAIN, 8));
	/*
		switch(columns) {
			case 0:
			case 1:
			...
		}
	*/
		
		g2.drawString(notePitch, 1, 9);
		g2.drawString("" + noteVelocity, 21, 9);
	}

	protected void calculateString(){

		noteVelocity = ""; 
		notePitch = "";
		if(note == null) return;

		int pitch = note.getPitch();
		switch (pitch % 12) {
			case  0: notePitch = "C";  break;
			case  1: notePitch = "C#"; break;
			case  2: notePitch = "D";  break;
			case  3: notePitch = "D#"; break;
			case  4: notePitch = "E";  break;
			case  5: notePitch = "F";  break;
			case  6: notePitch = "F#"; break;
			case  7: notePitch = "G";  break;
			case  8: notePitch = "G#"; break;
			case  9: notePitch = "A";  break;
			case 10: notePitch = "A#"; break;
			case 11: notePitch = "B";  break;
		}
		notePitch += pitch / 12;
		noteVelocity = ""+note.getVelocity();
	}

	public MooNote getNote(){
		return note;
	}

	class MAdapter extends MouseAdapter {
		public void mousePressed(MouseEvent e) {
			if (e.isControlDown()) {
				if (pitchRect.contains(e.getPoint())) {
					if (SwingUtilities.isRightMouseButton(e)) {
						note.setPitch(note.getPitch() + 1);
					} else if (SwingUtilities.isLeftMouseButton(e)) {
						note.setPitch(note.getPitch() - 1);
					}
					calculateString();
				} else if (veloRect.contains(e.getPoint())) {
					if (SwingUtilities.isRightMouseButton(e)) {
						note.setVelocity(note.getVelocity() + 1);
					} else if (SwingUtilities.isLeftMouseButton(e)) {
						note.setVelocity(note.getVelocity() - 1);
					}
					calculateString();
				}
				e.getComponent().repaint();
			}
			if (e.isPopupTrigger()) {
				popup.show(e.getComponent(), e.getX(), e.getY());
			}
		}
	}

	class PopupListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			Object source = e.getSource();
			if  (source == popupProp) {
				new MooDialog(note);
			} else if  (source == popupRemove) {
				remove();
			}
		}
	}

	protected void remove(){
		mtv.remove(this);
	}

}
