import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.sound.midi.*;

/**
 * The title bar for each track with track name, channelBox, instrument etc.
 * 
 * @author  Andersson, Andreen, Lanneskog, Pehrson
 * @version 1
 */
 
public class MooTrackTitle extends JPanel {

	private MetaMessage trackNameMessage;
	private String trackName = "";
	private ShortMessage programChangeMessage;
	private int channel = 0;
	private JTextField title;
	private MooInstrumentList instruments;
	private JComboBox channelBox;
	private JCheckBox mute;
	private JCheckBox solo;
	private Track track;
	private int trackNum;

	/** 
	 * Creates the title bar.
	 */
	public MooTrackTitle (Track aTrack, int aTrackNum) {
		setDoubleBuffered(true);
		track = aTrack;
		this.trackNum = aTrackNum;

		// Finds track name, program change and channel.
		MidiMessage msg;
		int status;
		for (int i = 0; i < track.size(); i++) {
			msg = track.get(i).getMessage();
			status = msg.getStatus();
			if (status == MetaMessage.META) {
				if (((MetaMessage)msg).getType() == 3) {
					trackNameMessage = (MetaMessage)msg;
					trackName = new String(trackNameMessage.getData());
				}
			} else if (status >= 192 && status <= 207) {
				programChangeMessage = (ShortMessage)msg;
				channel = status - 192;
			}
		}

/*		// Finds channel number.
		MidiEvent event;
		for (int i = 0; i < track.size(); i++) {
			event = track.get(i);
			if (event instanceof MooNote) {
				channel = ((MooNote)event).getChannel();
				break;
			}
		}
*/		
		// Creates and places components.
		setLayout(new GridLayout(4,1));
		setBorder(BorderFactory.createLineBorder(Color.black));

		setPreferredSize(new Dimension(MooTrackView.VIEW_WIDTH,70));
		title = new JTextField(trackName);
		title.setFont(Moosique.getGUI().FONT);
		title.addFocusListener(new TitleFocusListener());
		add(title);

		instruments = new MooInstrumentList(channel);
		// instruments = new MooInstrumentList(channel, programChangeMessage);
		add(instruments);

		channelBox = new JComboBox();
		channelBox.setFont(Moosique.getGUI().FONT);
		for (int i = 1; i <= 16; i++)
			channelBox.addItem(new Integer(i));

		channelBox.setSelectedIndex(channel);

		channelBox.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e){
					int chan = channelBox.getSelectedIndex();
					MidiEvent me;
					MooNote mn;
					instruments.setChannel(channelBox.getSelectedIndex());
					for (int j = 0; j < track.size(); j++) {
						me = track.get(j);
   					    if (me instanceof MooNote){
							mn = (MooNote)me;
							mn.setChannel(chan);
						}
    				}}});
		add(channelBox);

		JPanel checkboxes = new JPanel();
		checkboxes.setLayout(new GridLayout(1,3));

		mute = new JCheckBox("Mute");
		mute.setSelected(Moosique.getSequencer().getTrackMute(trackNum));
		mute.setFont(Moosique.getGUI().FONT);
		mute.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent event){
					boolean selected = mute.isSelected();
					if (selected){
						solo.setSelected(false);
						Moosique.getSequencer().setTrackSolo(trackNum, false);
					}
					Moosique.getSequencer().setTrackMute(trackNum, selected);

				}});
		checkboxes.add(mute);

		solo = new JCheckBox("Solo");
		solo.setSelected(Moosique.getSequencer().getTrackSolo(trackNum));
		solo.setFont(Moosique.getGUI().FONT);
		solo.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent event){
					//setSolo
					boolean selected = solo.isSelected();
					if (selected){
						mute.setSelected(false);
						Moosique.getSequencer().setTrackMute(trackNum, false);
					}
					Moosique.getSequencer().setTrackSolo(trackNum, selected);
				}});
		checkboxes.add(solo);
		add(checkboxes);
	}

	class TitleFocusListener extends FocusAdapter {
		public void focusLost(FocusEvent e) {
			// Update the MidiEvent containing the title of this track
		}
	}
}
