import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.sound.midi.*;
import java.util.*;

/**
 * Graphical representation of a MIDI track.
 * 
 * @author  Andersson , Andreen, Lanneskog, Pehrson
 * @version 1
 */

public class MooTrackView extends JPanel {

	private Track track;
	private Rectangle box;

	private JPopupMenu popup;
	private JMenuItem menuItem;
	private ArrayList rects;
	protected static int viewLength = 0;
	protected static int extraHeight = 0;
	public static final int NOTE_HEIGHT = 10, NOTE_WIDTH = 40, VIEW_WIDTH = 200;

	public MooTrackView (Track track) {
		super(true);
		this.track = track;

		extraHeight = Toolkit.getDefaultToolkit().getScreenSize().height - 150;
		// Configures panel
		setBackground(Color.white);
		setBorder(BorderFactory.createLineBorder(Color.black));
		setLayout(null);
		setPreferredSize(new Dimension(VIEW_WIDTH, 140 * NOTE_HEIGHT));

		// Creates temporary variables
		MidiEvent note;
		MooNoteElement elem;
		rects = new ArrayList(track.size() / 2);

		// Places note elements
		for (int i = 0; i < track.size(); i++) {
			note = track.get(i);
			if (note instanceof MooNote) {
				// Adds the note element to the note area.
				MooNote mn = (MooNote)note;
				elem = new MooNoteElement(this, mn);
				add(elem);

				layoutElement(elem,false);

				// Moves the note element to the appropriate place.
			}
			setPreferredSize(new Dimension(VIEW_WIDTH, viewLength + extraHeight));

		}
		validate();

		// Creates pop-up menu.
		popup = new JPopupMenu();
		menuItem = new JMenuItem("Add...");
		// menuItem.addActionListener();
		popup.add(menuItem);

		addMouseListener(new PopupListener());
	}

	public void layoutElement(MooNoteElement elem, boolean old){
		Rectangle r = new Rectangle();
		if(old){
			r =	elem.getBounds(r);
			for (Iterator i = rects.iterator(); i.hasNext();){
				Object ob = i.next();
				if (r.equals(ob)){
					rects.remove(ob);
					break;
				}
			}
		}

		int beatsPerSixteenth = Moosique.getSequence().getResolution() / 4;
		MooNote mn = elem.getNote();
		Insets insets = getInsets();
		int x, y, height;
		x = insets.left;
		y = insets.top + (int)(mn.getTick() / beatsPerSixteenth) * NOTE_HEIGHT;
		height = (mn.getDuration() / beatsPerSixteenth) * NOTE_HEIGHT;
		if (height == 0) height = NOTE_HEIGHT;
		r = new Rectangle(x, y, NOTE_WIDTH, height);
		while(isOccupied(r)) r.translate(NOTE_WIDTH, 0);
		elem.setBounds(r);
		rects.add(r);
		if (viewLength < (y + height)){
			viewLength = y + height;
			if(old)setPreferredSize(new Dimension(VIEW_WIDTH, viewLength + extraHeight));
		}

	}

	public Track getTrack() {
		return track;
	}


	/** 
	 * Updates the track view.
	 */
	public void update(long tickPosition) {
		repaint();
	}

	private boolean isOccupied(Rectangle r) {
		Iterator it = rects.iterator();
		while (it.hasNext()) {
			if(r.intersects((Rectangle)it.next())) return true;
		}
		return false;
	}
	
	public void remove(MooNoteElement elem) {
		remove((Component)elem);
		elem.getNote().removeFrom(track);
		validate();
		repaint();
	}

	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Graphics2D g2 = (Graphics2D)g;
		for (int c = 0; c < viewLength || c < getHeight(); c += NOTE_HEIGHT) {
			for (int r = 0; r < (10 * NOTE_WIDTH); r += NOTE_WIDTH) {
				box = new Rectangle(r, c, NOTE_WIDTH, NOTE_HEIGHT);
				g2.setColor(Color.gray);
				g2.draw(box);
			}
		}
	}

	class PopupListener extends MouseAdapter {
		public void mousePressed(MouseEvent e) {
			if (e.isPopupTrigger()) {
				popup.show(e.getComponent(), e.getX(), e.getY());
			}
		}
	}
}
